VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Easing"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' -------------------------------------------------------------------------------------
'
'   Easing class v0.1.b
'
'   written by Aleksandar Ruzicic
'
' -------------------------------------------------------------------------------------
'
'   you're free to use and abuse this in any way you like it!
'

Public Event onFrame(ByRef Value() As Single)
Public Event onStart(ByRef InitialValue() As Single, ByRef Cancel As Boolean)
Public Event onEnd(ByRef CurrentValue() As Single, Aborted As Boolean)

Public Enum EasingType
    Linear
    EaseIn
    EaseInOut
    EaseOut
    ExpoIn
    ExpoOut
    ExpoInOut
    BounceIn
    BounceOut
    BounceInOut
    ElasIn
    ElasOut
    ElasInOut
    BackIn
    BackOut
    BackInOut
    CubicIn
    CubicInOut
    CubicOut
End Enum


Private lCounter        As Long
Private lTimeStart      As Long
Private lTimeEnd        As Long
Private eType           As EasingType

Private t               As Single
Private d               As Single
Private b               As Single
Private c               As Single

Private fStart()        As Single
Private fEnd()          As Single
Private fCurrent()      As Single

Private lUpper          As Long
Private lLower          As Long

Private bRunning        As Boolean
'

' starts animation (and aborts previous if any)
Public Sub Start(EasingFormula As EasingType, StartValue() As Single, EndValue() As Single, Interval As Long)
    
    Dim bCancel As Boolean
    
    lUpper = UBound(StartValue)
    lLower = LBound(StartValue)
    
    If (lUpper <> UBound(EndValue)) Or (lLower <> LBound(EndValue)) Then
        Err.Raise vbObjectError + 3012, "Easing.Start", "Array bounds must be equal."
        Exit Sub
    End If
    
    bCancel = False
    RaiseEvent onStart(StartValue, bCancel)
    
    If Not bCancel Then
        
        Me.Abort
    
        lCounter = 0
        lTimeStart = 0
        lTimeEnd = Interval
        eType = EasingFormula
        
        fStart = StartValue
        fEnd = EndValue
        fCurrent = fStart
        
        d = lTimeEnd - lTimeStart
        
        bRunning = True
        
        While bRunning
            
            If ArraysAreEqual(fCurrent, fEnd) Then
                
                bRunning = False
                
                RaiseEvent onFrame(fEnd)
                RaiseEvent onEnd(fEnd, False)
            
            Else
            
                t = lCounter - lTimeStart
                
                RaiseEvent onFrame(NextFrame)
                
                lCounter = lCounter + 1
            
            End If
        
        Wend
        
    End If
    
End Sub
'

' aborts animation
Public Sub Abort()

    If bRunning Then
    
        RaiseEvent onEnd(fCurrent, True)
    
        bRunning = False
    
    End If
    
End Sub
'

' moves animation to next frame
Private Function NextFrame() As Single()
        
    Dim i   As Long
    Dim tmp As Single
    
    tmp = t ' t is usualy changed inside easing functions, so preserve it's value
    
    For i = lLower To lUpper
        
        b = fStart(i)
        c = fEnd(i) - fStart(i)
        t = tmp
        
        fCurrent(i) = GetValue(eType, fCurrent(i))
        
    Next
    
    NextFrame = fCurrent

End Function
'

' get function value
Private Function GetValue(e As EasingType, x As Single) As Single
    
    Dim tmp As Single
    
    Const PI As Single = 3.14159265358979
    
    Select Case e
        
        Case EasingType.Linear
            GetValue = c * t / d + b
        
        Case EasingType.EaseIn
            t = t / d
            GetValue = c * t ^ 2 + b
        
        Case EasingType.EaseInOut
            If t < d / 2 Then
                GetValue = 2 * c * t ^ 2 / d ^ 2 + b
            Else
                tmp = t - d / 2
                GetValue = -2 * c * tmp ^ 2 / d ^ 2 + 2 * c * tmp / d + c / 2 + b
            End If
        
        Case EasingType.EaseOut
            GetValue = -c * t ^ 2 / d ^ 2 + 2 * c * t / d + b
            
        Case EasingType.ExpoIn
            tmp = 1
            If c < 0 Then
                tmp = -1
                c = -c
            End If
            GetValue = tmp * Exp(Log(c) / d * t) + b
            
        Case EasingType.ExpoOut
            tmp = 1
            If c < 0 Then
                tmp = -1
                c = -c
            End If
            GetValue = tmp * (-Exp(-Log(c) / d * (t - d)) + c + 1) + b
    
        Case EasingType.ExpoInOut
            tmp = 1
            If c < 0 Then
                tmp = -1
                c = -c
            End If
            If t < d / 2 Then
                GetValue = tmp * (Exp(Log(c / 2) / (d / 2) * t)) + b
            Else
                GetValue = tmp * (-Exp(-2 * Log(c / 2) / d * (t - d)) + c + 1) + b
            End If
        
        Case EasingType.BounceIn
            t = d - t: tmp = b: b = 0
            GetValue = c - GetValue(BounceOut, x) + tmp
            
        Case EasingType.BounceOut
            t = t / d
            If t < 1 / 2.75 Then
                GetValue = c * (7.5625 * t ^ 2) + b
            ElseIf t < 2 / 2.75 Then
                t = t - 1.5 / 2.75
                GetValue = c * (7.5625 * t ^ 2 + 0.75) + b
            ElseIf t < 2.5 / 2.75 Then
                t = t - 2.25 / 2.75
                GetValue = c * (7.5625 * t ^ 2 + 0.9375) + b
            Else
                t = t - 2.625 / 2.75
                GetValue = c * (7.5625 * t ^ 2 + 0.984375) + b
            End If
        
        Case EasingType.BounceInOut
            tmp = b: b = 0
            If t < d / 2 Then
                t = t * 2
                GetValue = GetValue(BounceIn, x) * 0.5 + tmp
            Else
                t = t * 2 - d
                GetValue = GetValue(BounceOut, x) * 0.5 + c * 0.5 + tmp
            End If
        
        Case EasingType.ElasIn
            If t = 0 Then
                GetValue = b
            Else
                t = t / d
                If t = 1 Then
                    GetValue = b + c
                Else
                    tmp = (d * 0.3) / (2 * PI) * 0.785398163397448: t = t - 1
                    GetValue = -(c * 2 ^ (10 * t) * Sin((t * d - tmp) * (2 * PI) / (d * 0.3))) + b
                End If
            End If
        
        Case EasingType.ElasOut
            If t = 0 Then
                GetValue = b
            Else
                t = t / d
                If t = 1 Then
                    GetValue = b + c
                Else
                    tmp = (d * 0.3) / (2 * PI) * 0.785398163397448
                    GetValue = c * 2 ^ (-10 * t) * Sin((t * d - tmp) * (2 * PI) / tmp) + c + b
                End If
            End If
        
        Case EasingType.ElasInOut
            If t = 0 Then
                GetValue = b
            Else
                t = t / (d / 2)
                If t = 2 Then
                    GetValue = b + c
                Else
                    tmp = (d * 0.3) / (2 * PI) * 0.785398163397448: t = t - 1
                    If t < 1 Then
                        GetValue = -0.5 * (c * 2 ^ (10 * t) * Sin((t * d - tmp) * (2 * PI) / (d * 0.45))) + b
                    Else
                        GetValue = 1.70158 * 2 ^ (-10 * t) * Sin((t * d - 1.70158) * (2 * PI) / (d * 0.45)) * 0.5 + c + b
                    End If
                End If
            End If
            
        
        Case EasingType.BackIn
            t = t / d
            GetValue = c * t ^ 2 * (2.70158 * t - 1.70158) + b
        
        Case EasingType.BackOut
            t = t / d - 1
            GetValue = c * (t ^ 2 * (2.70158 * t + 1.70158) + 1) + b
        
        Case EasingType.BackInOut
            t = t / (d / 2)
            If t < 1 Then
                GetValue = c / 2 * (t ^ 2 * (3.5949095 * t - 2.5949095)) + b
            Else
                t = t - 2
                GetValue = c / 2 * (t ^ 2 * (3.5949095 * t + 2.5949095) + 2) + b
            End If
        
        Case EasingType.CubicIn
            t = t / d
            GetValue = c * t ^ 3 + b
        
        Case EasingType.CubicOut
            t = t / d - 1
            GetValue = c * (t ^ 3 + 1) + b
        
        Case EasingType.CubicInOut
            t = t / (d / 2)
            If t < 1 Then
                GetValue = c / 2 * t ^ 3 + b
            Else
                t = t - 2
                GetValue = c / 2 * (t ^ 3 + 2) + b
            End If
            
    End Select
    
End Function
'

Private Function ArraysAreEqual(a1() As Single, a2() As Single) As Boolean
    
    Dim i As Long
    
    For i = lLower To lUpper
        If a1(i) <> a2(i) Then Exit Function
    Next
    
    ArraysAreEqual = True

End Function
